$(document).ready(function () {
  const puzzles = $('.puzzle');
  if (puzzles.length > 0) {
    for (let i = 0; i < puzzles.length; i++) {
      const id = $(puzzles[i]).attr('data-game-data-var');
      puzzleInit(puzzles[i], id);
    }
  }
});

function puzzleInit(puzzleBoard, puzzleId) {
  const gameData = window[puzzleId];

  const slotsDiv = $('<div class="slots">');
  const piecesDiv = $('<div id="pieces">');
  const endPrompt = $('<div>', {
    class: 'prompt',
    html: '<div>Slika je uspje\u0161no slo\u017eena. Bravo!</div>',
  });

  let activePiece;

  let pieces;
  let slots;

  const image = new Image();
  image.src = gameData.image;
  image.onload = function () {
    const factor = setDimensions(image);

    createPieces(image, puzzleBoard, gameData.size, factor, function (piecesArray) {
      pieces = piecesArray;

      shuffle(pieces);

      for (let i = 0; i < pieces.length; i++) {
        $(pieces[i]).appendTo(piecesDiv);
      }
    });
    createSlots(image.width * factor / gameData.size.x, image.height * factor / gameData.size.y, function (slotsArray) {
      slots = slotsArray;
    });

    image.width = image.width * factor;
    image.height = image.height * factor;
  }

  slotsDiv.appendTo(puzzleBoard);
  piecesDiv.appendTo(puzzleBoard);

  slotsDiv.on('pointerdown', function () {
    if (activePiece) {
      removeActive(activePiece);
    }
  });
  
  /*BT rješenje*/
  //var helpbutton = $("<button>", { "class" : "button my-4 fullpuzzle", text: "Rješenje slagalice", "onClick": "showHide('"+puzzleId+"')"});
  var helpbutton = $("<div><button class='button my-4 fullpuzzle' onClick='showHide(&quot;"+puzzleId+"&quot;)'>Rješenje slagalice</button></div>");
  var helpcontianer = $("<div>", {id: puzzleId, style: "display:none"});
  var helpslika = $("<img>", {src: image.src, style:"max-height: 560px;", alt: "Rješenje slagalice"});
  helpbutton.appendTo(puzzleBoard);
  helpslika.appendTo(helpcontianer);
  helpcontianer.appendTo(puzzleBoard);						  

  $(puzzleBoard).droppable({
    drop: function (e, ui) {
      revertElement(piecesDiv, ui);
      setTimeout(function () { $(ui.draggable[0]).draggable('option', 'revert', false); }, 500); // timeout because no current droppable/draggable callbacks is good enough for this
      removeActive(activePiece);
    }
  });

  function createPieces(image, board, size, factor, callback) {
    const px = 'px';

    const x = size.x;
    const y = size.y;

    const puzzlePieceWidth = image.width * factor / x;
    const puzzlePieceHeight = image.height * factor / y;

    const puzzlePieces = [];

    for (let i = 0; i < y; i++) {
      for (let j = 0; j < x; j++) {
        const puzzlePiece = $('<div>', {
          class: 'piece',
          x: j,
          y: i,
          css: {
            width: puzzlePieceWidth + px,
            height: puzzlePieceHeight + px,
            'background-image': 'url(' + image.src + ')',
            backgroundSize: image.width * factor + px + ' ' + image.height * factor + px,
            backgroundRepeat: 'no-repeat',
            'background-position-x': '-' + (j * puzzlePieceWidth) + px,
            'background-position-y': '-' + (i * puzzlePieceHeight) + px
          }
        });

        puzzlePiece.on('pointerdown', function (e) {
          clickPiece(this);
        });

        puzzlePiece.draggable({
          containment: board,
        });

        puzzlePieces.push(puzzlePiece);
        puzzlePiece.appendTo(pieces);
      }
    }
    callback(puzzlePieces);
  }

  function createSlots(width, height, callback) {
    const table = $('<table>');
    const puzzleSlots = [];
    for (let i = 0; i < gameData.size.y; i++) {
      const row = $('<tr>');
      for (let j = 0; j < gameData.size.x; j++) {
        const td = $('<td>');
        const slot = $('<div>', {
          class: 'slot',
          height: height,
          width: width,
          x: j,
          y: i
        });

        slot.on('pointerdown', function (e) {
          e.stopPropagation();
          clickSlot(this);
        });

        slot.droppable({
          tolerance: 'pointer',
          greedy: true,
          drop: function (e) {
            e.stopPropagation();
            clickSlot(e.target);
          }
        });

        slot.appendTo(td);
        puzzleSlots.push(slot);
        td.appendTo(row);
      }
      row.appendTo(table);
    }
    table.appendTo(slotsDiv);

    callback(puzzleSlots);
  }

  function checkIfSolved(pieces) {
    if (pieces.children().length > 0) {
      return;
    }

    const slotDivs = slotsDiv.find('.slot');
    const puzzleDivs = slotsDiv.find('.piece');

    for (let i = 0; i < slotDivs.length; i++) {
      if ($(slotDivs[i]).attr('x') == $(puzzleDivs[i]).attr('x') && $(slotDivs[i]).attr('y') == $(puzzleDivs[i]).attr('y')) {
        $.noop();
      } else {
        return;
      }
    }

    slotsDiv.detach();
    pieces.append(endPrompt);

    image.width = image.width;
    image.height = image.height;
    //$(image).appendTo(puzzleBoard);
    $(image).prependTo(puzzleBoard);
  }

  function makeActive(el) {
    el.classList.add('selected');
    activePiece = el;
  }

  function removeActive(el) {
    if (!el) {
      return;
    }
    el.classList.remove('selected');
    activePiece = undefined;
  }

  function switchPos(el1, el2) {
    const parent1 = $(el1).parent();
    const parent2 = $(el2).parent();

    $(el1).detach();
    $(el2).detach();

    parent1.append(el2);
    parent2.append(el1);

    $(el1).css({
      left: 0,
      top: 0,
    });
    $(el2).css({
      left: 0,
      top: 0,
    });

    checkIfSolved(piecesDiv);
  }

  // EVENTS

  function clickPiece(piece) {
    const isSlotted = $(piece).parent('.slot');
    if ($(piece).parent('.slot')) {
      event.stopPropagation();
    }

    if (activePiece) {
      if (activePiece === piece) {
        removeActive(activePiece);
        return;
      } else if (isSlotted) {
        switchPos(activePiece, piece);
        removeActive(activePiece);
        return;
      }
      removeActive(activePiece);
    }
    makeActive(piece);
  }

  function clickSlot(slot) {
    if (!activePiece) {
      return;
    }
    if ($(slot).children().length) {
      switchPos($(slot).children()[0], activePiece);
    } else {
      $(activePiece).detach();
      $(activePiece).appendTo(slot);
      $(activePiece).css({
        left: 0,
        top: 0,
      });
    }
    removeActive(activePiece);
    checkIfSolved(piecesDiv);
  }
}

// UTILS

function shuffle(a) {
  var j, x, i;
  for (i = a.length - 1; i > 0; i--) {
    j = Math.floor(Math.random() * (i + 1));
    x = a[i];
    a[i] = a[j];
    a[j] = x;
  }
  return a;
}

function revertElement(originalContainer, ui) {
  const el = $(ui.draggable[0]);
  if (el.parent()[0] !== originalContainer[0]) {
    el.appendTo(originalContainer);
    el.css({
      top: 0,
      left: 0
    });
  } else {
    el.data('uiDraggable').originalPosition = {
      top: 0,
      left: 0
    };
    el.draggable({
      revert: true,
      revertDuration: 0,
    });
  }
}

function setDimensions(image) {
  const ratio = image.width / image.height;
  const scrwidth = jQuery(window).width();
  let newWidth;
  let newHeight;

  if (image.height > image.width) {
    newHeight = 480;
    newWidth = newHeight * ratio;

    if (newWidth > scrwidth - 60) {
      newWidth = scrwidth - 60;
      newHeight = newWidth / ratio;
    }
  } else {
    newWidth = 640;

    if (newWidth > scrwidth - 60) {
      newWidth = scrwidth - 60;
      newHeight = newWidth / ratio;
    }
  }

  const factor = (newWidth / image.width);

  return factor;
}